<?php
/**
 * Tree-Nation
 * Offer a tree each time a customer buys a product. 1 tree will be offered for each product purchase. The user will receive an email with his trees offered by your brand.
 * php version 7.2.27
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the tree-nation.com license that is
 * available through the world-wide-web at this URL:
 * https://tree-nation.com/legal/privacy-policy

 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category  Tree-Nation
 * @package   Treenation_Webhook
 * @author    Tree-Nation <auto-plant@tree-nation.com>
 * @copyright 2020 Tree-Nation
 * @license   https://www.gnu.org/licenses/gpl-3.0.txt GNU/GPLv3
 * @link      https://tree-nation.com
 * @see       https://tree-nation.com
 */

namespace Treenation\Webhook\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Sales\Model\Order;

/**
 * Class OrderObserver
 *
 * @package Treenation\Webhook\Observer
 */
class OrderObserver implements ObserverInterface
{
    protected $logger;
    protected $enabled;
    protected $treenation_id;
    protected $treenation_url;

    /**
     * OrderObserver constructor.
     *
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(\Psr\Log\LoggerInterface $logger)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $config = $objectManager->get('Magento\Framework\App\Config\ScopeConfigInterface');
        $enabled = $config->getValue('tn_webhook/general/enabled');
        $value1 = $config->getValue('tn_webhook/general/treenation_id');
        $value2 = $config->getValue('tn_webhook/general/advanced_parameters');

        $this->enabled = $enabled;
        $this->treenation_id = $value1;
        $this->treenation_url = $value2;

        $this->logger = $logger;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!$this->enabled) {
            return;
        }

        $order = $observer->getOrder();
        $this->logger->debug('order was saved with status: '.json_encode($order->getState()));

        if ($order->getState() == Order::STATE_COMPLETE) {
            $this->hookActionPaymentConfirmation($order);
        }

    }

    /**
     * @param Order $order
     */
    public function hookActionPaymentConfirmation(Order $order)
    {
        $this->logger->info('order status is COMPLETE. Sending hook!');
        $this->logger->info('url: '.$this->treenation_url.' | tn id: '.$this->treenation_id);

        $this->logger->info($order->getCustomerEmail());

        $products = $order->getAllItems();
        $webhookdata = [
            'cUrl' => $this->treenation_url ? : 'default', // field needed only for debug purposes
            'treenation_id' => $this->treenation_id,
            'order_id' => $order->getRealOrderId(),
            'email' => $order->getCustomerEmail(),
            'name' => $order->getCustomerName(),
            'order_total' => $order->getTotalPaid(),
            'order_products' => [],
        ];
        foreach ($products as $product) {
            $webhookdata['order_products'][] = [
                'product_id' => $product->getProductId(),
                'price' => $product->getPrice(),
                'quantity' => $product->getQtyInvoiced(),
            ];
        }
        $this->logger->info('payment hook was called. webhook data to send: '.json_encode($webhookdata));
        // send curl to configured tn url
        $result = $this->_sendCurl($webhookdata);

        // log result
        if ($result['success'] == true) {
            $this->logger->info('payment hook was sent. response data was: '.$result['success']);
        } else {
            $this->logger->error('payment hook was sent. cUrl ERROR: '.$result['error']);
        }

    }

    /**
     * @param  $data
     * @return array
     */
    private function _sendCurl($data)
    {
        $url = $this->treenation_url ? $this->treenation_url : 'https://tree-nation.com/magento-webhook';
        $fields = $data;

        //url-ify the data for the POST
        $fields_string = http_build_query($fields);
        //open connection
        $ch = curl_init();
        //set the url, number of POST vars, POST data
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        //execute post
        $result = [
            'success' => curl_exec($ch),
            'error' => '',
        ];
        if ($result['success'] != true) {
            $result['error'] = curl_error($ch);
        }

        curl_close($ch);

        return $result;
    }
}
